class "Level"{
    x_pos   =   0,
    y_pos   =   0,
    state   =   ""
}

function Level:__init__(filename, p_world)
    assert( filename ~= nil )
    -- load table from file
    local info,err = table.load( filename )
    assert( err == nil )
    info.imagename  =   info.imagename:match("([^/]+)$") or info.imagename
    self.image      =   love.graphics.newImage("res/images/level/"..info.imagename)
    self.y_pos      =   0
    self.x_pos      =   0
    self.polygon    =   info.collidable or nil
    self.height     =   self.image:getHeight()
    self.width      =   self.image:getWidth()
    self.nutriments =   {}

    --If a world is passed, we go for a physics simulation
    if p_world then
        self.world  =   p_world
        levelshape  =   love.physics.newChainShape( false, unpack(info.collidable) )
        levelbody   =   love.physics.newBody(p_world, 0,0, "static")
        levelfixt   =   love.physics.newFixture(levelbody, levelshape)
        --debug?
        self.shape  =   levelshape
        self.body   =   levelbody
    end
end

function Level:update(p_dt)
    --~ TODO:Ground:update?
end

function Level:getDrawable()
    return self.image, self.x_pos, self.y_pos
end

function Level:draw()
    love.graphics.draw(self:getDrawable())

end

-- Returns true if the point is underground
function Level:isUnderground(p_x, p_y)
    --Find closest segment
    for i=1, #self.polygon, 2 do
        if self.polygon[i] >= p_x then -- Finding the good point of the surface
            local horizon    =   self.polygon[i+1] -- Getting Y value of the ground here...
            if i>2 and horizon > self.polygon[i-1] then -- Testing wich is the lowest (Y) with the previous point
                horizon      =   self.polygon[i-1]
            end

            return (p_y < horizon)
        end
    end
end
