------------------------------------------------------------------------
-- MAINMENU - allows the player to start or load a game and such
------------------------------------------------------------------------

require("class.Rectangle")

mainmenu                =   Gamestate.new()
mainmenu.position       =   vector(love.graphics.getWidth()/2, love.graphics.getHeight()/2)
mainmenu.rotation       =   0
mainmenu.zoom           =   0.4
mainmenu.items          =   {}



-- Adding branch buttons
optionitem  =   ImageButton()
optionitem:loadImage("res/images/gui/main_menu/branch_right.png")
optionitem:loadLabelImage("res/images/gui/main_menu/label_options.png")
optionitem:setCenter(1005, 122)
optionitem:loadHoverSound("res/sounds/button_onHover.wav")
optionitem:loadClickSound("res/sounds/button_onClick.wav")

function optionitem:onClick()
    self:click()
    TRANSITION:goTo(optionmenu)
end
table.insert(mainmenu.items, optionitem)

playitem  =   ImageButton()
playitem:loadImage("res/images/gui/main_menu/branch_left.png")
playitem:loadLabelImage("res/images/gui/main_menu/label_play.png")
playitem:setCenter(-7, 187)
playitem:loadHoverSound("res/sounds/button_onHover.wav")
playitem:loadClickSound("res/sounds/button_onClick.wav")
function playitem:onClick()
    self:click()
    TRANSITION:goTo(newgame)
end
table.insert(mainmenu.items, playitem)

continueitem  =   ImageButton()
continueitem:loadImage("res/images/gui/main_menu/branch_top.png")
continueitem:loadLabelImage("res/images/gui/main_menu/label_continue.png")
continueitem:setCenter(463, -97)
continueitem:loadHoverSound("res/sounds/button_onHover.wav")
continueitem:loadClickSound("res/sounds/button_onClick.wav")
function continueitem:onClick()
    self:click()
    print("Not yet")
end
table.insert(mainmenu.items, continueitem)

exititem  =   ImageButton()
exititem:loadImage("res/images/gui/main_menu/branch_bottom.png")
exititem:loadLabelImage("res/images/gui/main_menu/label_quit.png")
exititem:setCenter(504, 750)
exititem:loadHoverSound("res/sounds/button_onHover.wav")
exititem:loadClickSound("res/sounds/button_onClick.wav")
function exititem:onClick()
    self:click()
    love.event.quit()
end
table.insert(mainmenu.items, exititem)


mainmenu.hud  =   {}

statsbutton   =   MenuItem("Stats")
statsbutton:setPos(love.graphics.getWidth() - statsbutton.rect.width, 10 )
function statsbutton:onClick()
    print("Stat clicked")
    TRANSITION:goTo(statmenu)
end
table.insert(mainmenu.hud, statsbutton)


function mainmenu:init()

end

function mainmenu:enter(previous)
    print("Entering main menu")
end

function mainmenu:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    TRANSITION:update(dt)
    m_x, m_y = menucam:mousepos()
    for i, button in pairs(mainmenu.items) do
        if button.rect:inflate(-420, -320):contains(m_x, m_y) then
            button.hover    =   true
        else
            button.hover    =   false
        end
        button:update(dt)
    end

    x, y    =   love.mouse.getPosition()
    for i, hudelt in pairs(mainmenu.hud) do
        hudelt:update(dt)
        if hudelt:contains(x, y) then
            hudelt:onHover()
        end
    end
end

function mainmenu:draw()
    menucam:attach()
    if background then
        love.graphics.draw(background.image, background.pos:unpack())
    end
    for i, button in pairs(mainmenu.items) do
        button:draw()
    end

    menucam:detach()

    for i, hudelt in pairs(mainmenu.hud) do
        hudelt:draw()
    end
    if DEBUG then
        debug_draw_stats(menucam)
    end
    watermark()
end

function mainmenu:keyreleased(key, unicode)
    if key=="return" then
        Gamestate.switch(newgame)
    end
end

function mainmenu:mousereleased(m_x, m_y, button)
    local x, y = menucam:mousepos()
    for i, button in pairs(mainmenu.items) do
        if button.rect:contains(x,y) then
            button:onClick()
        end
    end

    for i, hudelt in pairs(mainmenu.hud) do
        if hudelt:contains(m_x, m_y) then
            hudelt:onClick()
        end
    end
end

