------------------------------------------------------------------------
-- NEW GAME
------------------------------------------------------------------------
newgame                 =   Gamestate.new()
newgame.position        =   vector(-75, 170)
newgame.rotation        =   0
newgame.zoom            =   1.5


function newgame:init() -- run only once
    print("Entering NEW GAME state")
    print("Loading seed images")
    species =   load_species()
    angle   =   0
    rb  =   ringbuffer(0)
    for k,v in pairs(species) do
        rb:insert(v)
    end
    rb:remove()


    -- Creating buttons
    h = love.graphics.getHeight()/2
    w = love.graphics.getWidth()/2
    padding = 200

    l = love.graphics.newImage("res/images/sprites/left-arrow.png") -- Default image
    r = love.graphics.newImage("res/images/sprites/right-arrow.png") -- Default image
    right_arrow =   Button()
    right_arrow:setImage(r)
    right_arrow:setCenter(w + padding, h)
    right_arrow.onClick = function() rb:next() end

    left_arrow  =   Button()
    left_arrow:setImage(l)
    left_arrow:setCenter(w - padding, h)
    left_arrow.onClick = function() rb:prev() end

    play_button         =   {}
    play_button.rect    =   Rectangle(0,0,150,45)
    play_button.rect:setCenter(w, h + padding*1.5)
    play_button.label   =   "Play"

    font = love.graphics.newFont("res/fonts/Varling.ttf", 35)
    love.graphics.setFont(font)
    preview_alpha    =   0
    print("Choosing menu is ready")
end

function newgame:enter()
    preview_alpha    =   0
end

function newgame:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    TRANSITION:update(dt)
    angle   =   angle + 0.5 * dt


    preview_alpha   =   preview_alpha + 300 * dt
    if preview_alpha > 255 then preview_alpha = 255 end
end

function newgame:enter(previous) -- run every time the state is entered
    menucam.rot         =   math.rad(newgame.rotation)
end

function newgame:draw()
    menucam:attach()
    --Drawing splash image
    love.graphics.draw(background.image, background.pos:unpack())

    menucam:detach()

    -- If we have a preview image: draw it!
    if rb:get().preview_image then
        love.graphics.setColor(255, 255, 255, preview_alpha)
        local w =   rb:get().preview_image:getWidth()
        local h =   rb:get().preview_image:getHeight()
        love.graphics.draw(rb:get().preview_image, love.graphics.getWidth() - w)
        love.graphics.setColor(255, 255, 255, 255)
    end

    mid_x = love.graphics.getWidth()  / 2
    mid_y = love.graphics.getHeight() / 2


    rect_w= 256
    rect_h= 256

    -- Draw the frame
    love.graphics.setColor(10, 155, 10, 120)
    love.graphics.rectangle( "fill", mid_x - rect_w/2, mid_y - rect_h/2, rect_w, rect_h )

    love.graphics.setColor(55, 55, 55, 100)
    rect_w=rect_w/1.025
    rect_h=rect_h/1.025
    love.graphics.rectangle( "fill", mid_x - rect_w/2, mid_y - rect_h/2, rect_w, rect_h )


    -- Draw the image of the seed
    rect_w= rb:get().seed_image:getWidth()
    rect_h= rb:get().seed_image:getHeight()
    love.graphics.setColor(255, 255, 255, preview_alpha)
    love.graphics.draw(rb:get().seed_image, mid_x, mid_y, angle, 1, 1, rect_w/2, rect_h/2)
    love.graphics.setColor(255, 255, 255, 255)

    -- Draw the buttons
    love.graphics.draw(left_arrow:getDrawable())
    love.graphics.draw(right_arrow:getDrawable())
    love.graphics.rectangle(play_button.rect:getDrawable())
    love.graphics.printf(_(play_button.label), play_button.rect.x, play_button.rect.y, play_button.rect.width, "center")


    -- Draw the text
    love.graphics.setFont(title_font)
    love.graphics.printf(_(rb:get().name), 0, 80, love.graphics.getWidth(), "center")
    love.graphics.setFont(standard_font)
    desc_string =   _("Description")..":\t".._(rb:get().description).."\n"
    .._("Jumps")..":\t"..rb:get().max_jumps.."\n"
    .."Max branches:\t"..rb:get().max_branches .."\n"
    .."Max roots:\t"..rb:get().max_roots

    love.graphics.printf(_(desc_string), love.graphics.getWidth()/4, 540, love.graphics.getWidth()/2, "left")

    watermark()
    if DEBUG then debug_draw_stats(menucam) end

end

function newgame:keyreleased(key)
    if key == "right" then
        preview_alpha = 0
        rb:next()
    end

    if key == "left" then
        preview_alpha = 0
        rb:prev()
    end

    if key == "escape" then
        TRANSITION:goTo(mainmenu)
    end

    if key == "return" then
        print("entering game...")
        print("Chosen species: "..rb:get().name)
        seeding.species =   rb:get()
        Gamestate.switch(seeding)
    end
end

function newgame:mousereleased(m_x, m_y, button)
    print("click at: "..m_x.." "..m_y)
    if button == "l" then
        if left_arrow:contains(m_x, m_y) then
            preview_alpha = 0
            rb:prev()
        elseif right_arrow:contains(m_x, m_y) then
            preview_alpha = 0
            rb:next()
        elseif play_button.rect:contains(m_x, m_y) then
            seeding.species =   rb:get()
            Gamestate.switch(seeding)
        end

    end

end
