------------------------------------------------------------------------
-- SOUND_OPTION - let's choose thoses graphical options
------------------------------------------------------------------------

soundoption              =   Gamestate.new()
soundoption.position     =   vector(1650, 340)
soundoption.rotation     =   0
soundoption.zoom         =   1.5


function soundoption:enter()
    print("Entering sound options!")
    tprint(GAMECONFIG.options)
    -- Creating list of items
    item_list   =   {"Music", "Sound", "Volume", "Back"}
    value_list  =   {GAMECONFIG.options["music"], GAMECONFIG.options["sound"], GAMECONFIG.options["music_vol"], nil}

    line_offset =   (love.graphics.getHeight()/2) / #item_list
    line_offset =   line_offset -   big_font:getHeight()

    menu        =   {}

    ypos  =   love.graphics.getHeight()/2
    -- Building the menu items
    for k,v in pairs(item_list) do
        local item  =   MenuItem( item_list[k], value_list[k])
        local xpos  =   love.graphics.getWidth()/2 - item.rect.width/2
        item:setPos( xpos, ypos )
        ypos = ypos+line_offset

        function item:onClick()
            print("Clicked on "..self.label)
            self:click()
            if item.label=="Sound" then
                GAMECONFIG:toggle("sound")
                self.value  =   GAMECONFIG.options["sound"]
                GAMECONFIG:saveToFile()

            elseif item.label=="Music" then
                GAMECONFIG:toggle("music")
                self.value  =   GAMECONFIG.options["music"]
                GAMECONFIG:saveToFile()

            elseif item.label=="Volume" then
                local vol = GAMECONFIG.options["music_vol"] +0.1
                if vol > 1.0 then vol = 0 end
                GAMECONFIG.options["music_vol"] = vol
                self.value  =   vol
                GAMECONFIG:saveToFile()

            elseif item.label=="Back" then
                TRANSITION:goTo(optionmenu)
            end
        end
        table.insert(menu, item)
    end

end

function soundoption:init()
    --Instanciate options items


end

function soundoption:update(dt)
    menucam:move(math.sin(gametime)/8,math.cos(gametime*0.5)/8)
    TRANSITION:update(dt)
    for k, item in pairs(menu) do
        item:update(dt)

        x, y = love.mouse.getPosition()
        if item:contains(x, y) then
            item:onHover()

        end
    end
end

function soundoption:keyreleased(key, unicode)
    if key=="esc" then
        TRANSITION:goTo(optionmenu)
    end
end

function soundoption:mousereleased(m_x, m_y, button)
    for k, item in pairs(menu) do
        if item:contains(m_x, m_y) then
            print("click on item")
            item:onClick()
        end
    end
end

function soundoption:draw()
    menucam:attach()
    --Drawing back image
    love.graphics.draw(background.image, background.pos:unpack())
    menucam:detach()

    love.graphics.setFont(big_font)
    for k, item in ipairs(menu) do
        item:draw()

    end

    if DEBUG then debug_draw_stats(menucam) end
    watermark()

end
