------------------------------------------------------------------------
-- @file play_gamestates.lua
-- File storing the differents gamestates of ingame modes and their callbacks
------------------------------------------------------------------------

seeding =   Gamestate.new()
growing =   Gamestate.new()

--Drawing groups
GAME_OBJECTS    =   {}
PARTICLES       =   {}
ENV_OBJECTS     =   {}
GUI             =   {}
world           =   {}


--Transition management
seeding.zoom    =   1
growing.zoom    =   0.5
------------------------------------------------------------------------
-- SEEDING
------------------------------------------------------------------------
function seeding:init()
    TRANSITION.TIME     =   1.2     --Redifining the transition duration for this state
    seedcam             =   camera(0, 0, 1, 0) -- Camera at origin point, with no zoom
    seeding.speedcam    =   10


    --TODO: refactor this
    sky                 =   love.graphics.newImage("res/images/level/sky.png")

    -- Physics
    love.physics.setMeter(64)
    world = love.physics.newWorld(0, 7.91*64, true)

    -- Initing our Seed
    --TODO:refactor this
    species 		=   seeding.species
    seed    		=   Seed(10, 10)
    seed.species    =   species
    seed:setImage(species.seed_image)
    seed:setJumpLeft(species.max_jumps)
    seed:setShape(species.seed_shape)
    seed:setWorld(world)
    seed:setEffectDuration(TRANSITION.TIME)

    -- For a bit of randomness, add an impule in the beginning
    local rand 	=   math.random()
    rand        =   rand+0.5
    seed.body:applyLinearImpulse( 75 * rand, -75 * rand )
    rand            =   nil
    current_level   =   Level("data/levels/level_0", world)
    table.insert(ENV_OBJECTS, current_level)
    table.insert(GAME_OBJECTS, seed)


    --~ Register particles effects
    table.insert(PARTICLES, seed.jump_effect)
    table.insert(PARTICLES, seed.creation_effect)

    --TODO: refactor this: nutriment must be relative to level data
    nut = nil
    for i = 0, 200, 1 do
        local nut	=	Nutriment()
        local x     =   math.random(current_level.polygon[1], current_level.polygon[#current_level.polygon - 1])
        local y 	=	math.random(0, current_level.height)

        table.insert(ENV_OBJECTS, nut)
        while current_level:isUnderground(x, y) do
            x   =   math.random(current_level.polygon[1], current_level.polygon[#current_level.polygon - 1])
            y   =   math.random(0, current_level.height)
        end
        nut:setPosition(x, y)
        table.insert(current_level.nutriments, nut)
    end

    -- TODO: make it the borders of the level
    screenright         =   love.graphics.getHeight()
    screenright         =   love.graphics.getWidth()
    -- TODO: make the level editor generate borders dynamically
    border_rectangle    =   love.physics.newRectangleShape(10, current_level.image:getHeight() )

    border_left         =   {}
    border_left.body    =   love.physics.newBody(world, 0, current_level.image:getHeight()/2, "static") --Anchor point
    border_left.fixture =   love.physics.newFixture(border_left.body, border_rectangle)

    border_right        =   {}
    border_right.body   =   love.physics.newBody(world,current_level.image:getWidth(), current_level.image:getHeight()/2, "static") --Anchor point
    border_right.fixture=   love.physics.newFixture(border_right.body, border_rectangle)


end

function seeding:enter()
    love.mouse.setGrab(true)

end

function seeding:draw()
    love.graphics.draw(sky, 0, 0)

    love.graphics.setFont(standard_font)
    --Drawing _moving_ world
    seedcam:attach()
    for idx,obj in ipairs(ENV_OBJECTS) do
        obj:draw()
    end
    for idx,obj in ipairs(GAME_OBJECTS) do
        obj:draw()
    end


    for idx,p in ipairs(PARTICLES) do
        love.graphics.draw(p, 0, 0)
    end

    -- INFO: keep that as an example: don't fall into that bug again:
    -- body: getPosition returns the FRACKING CENTER of the shape, not it's DARN TOPLEFT CORNER
    --~ x, y = border_left.body:getWorldCenter()
    --~ love.graphics.rectangle("fill", x, y - current_level.image:getHeight()/2, 10, current_level.image:getHeight() )

    --~ x, y = border_right.body:getPosition()
    --~ love.graphics.rectangle("fill", x, y - current_level.image:getHeight()/2, 10, current_level.image:getHeight() )

    --~ DEBUG
    if DEBUG then
        debug_draw_shapes(GAME_OBJECTS)

    end

    --Drawing _fixed_ HUD
    seedcam:detach()

    watermark()
    for i, gui in pairs(GUI) do
        gui:draw()
    end
    if DEBUG then
        debug_draw_stats(seedcam)
    end
    love.graphics.print( _("You have ")..seed.jumps.._(" jumps remaining."), 10, 10 )

end

function seeding:update(dt)
    m_x, m_y    =   love.mouse.getPosition()
    world:update(dt) --Physic world


    for idx,obj in ipairs(ENV_OBJECTS) do
        obj:update(dt)
    end

    --Player parts
    for idx,obj in ipairs(GAME_OBJECTS) do
        obj:update(dt)
    end

    --Particle effects
    for idx,p in ipairs(PARTICLES) do
        p:update(dt)
    end

    local point =   {}
    point.x, point.y=seed.x_pos, seed.y_pos
    track_point(seedcam, point)


    for idx,gui in ipairs(GUI) do
        gui:update(dt)
        if not gui:isAlive() then
            table.remove(GUI, idx)
        elseif gui:testPoint(m_x, m_y) then
            gui:onHover(m_x, m_y)
        end
    end


    if seed.planted then
        if not zoom_factor then
            zoom_factor =   (growing.zoom - seeding.zoom)/TRANSITION.TIME
        end
        seedcam.zoom    =   seedcam.zoom + zoom_factor*dt
    end
    if seed:isPlaced() then
        Gamestate.switch(growing)
    end

end

function seeding:keypressed(key, unicode)

    if key == "escape" then
        love.event.quit()
    end

    if key == " " then --spacebar pressed
        print("planting!")
        seed:plant()
    end

    if key == "right" then
        seed:jump("right")
    end

    if key == "left" then
        seed:jump("left")
    end

end

function seeding:mousereleased(m_x, m_y, button)
    x,y =   seedcam:mousepos()
    click_handled   =   false

    for i,gui in ipairs(GUI) do
        if gui:onClick(m_x,m_y) then
            click_handled   =   true
        end
    end
    if not click_handled then
        if seed.fixture:testPoint(x, y) then
            radial    =   RadialMenu(seed)
            radial:setItems(seed:getActions())
            radial:setPosition(m_x,m_y)
            table.insert(GUI, radial)
            radial    =   nil
            --~ seed:plant()
        else
            if x<seed.x_pos then
                seed:jump("left")
            else
                seed:jump("right")
            end
        end
    end
end

------------------------------------------------------------------------
-- GROWING
------------------------------------------------------------------------
function growing:init()
    player  =   Tree(species)
    player:setCore(seed)
    player:setCamera(seedcam)

end

function growing:enter()
    print("Entering growing mode...")
    love.mouse.setGrab(true)
    seedcam.zoom    =   growing.zoom

end

function growing:update(dt)
    world:update(dt)
    m_x, m_y    =   love.mouse.getPosition()

    -- Scroll management (when mouse hits the borders) or arrows used
    if m_x == 0 or love.keyboard.isDown("left") then
        seedcam:move(-GAMECONFIG.options["scroll_speed"], 0)
    end
    if m_x == love.graphics.getWidth() -1 or love.keyboard.isDown("right") then
        seedcam:move(GAMECONFIG.options["scroll_speed"], 0)
    end
    if m_y == 0 or love.keyboard.isDown("up") then
        seedcam:move(0, -GAMECONFIG.options["scroll_speed"])
    end
    if m_y == love.graphics.getHeight() -1 or love.keyboard.isDown("down") then
        seedcam:move(0, GAMECONFIG.options["scroll_speed"])
    end


    for idx,obj in ipairs(ENV_OBJECTS) do
        obj:update(dt)
        if obj.dead then
            table.remove(ENV_OBJECTS, idx)
        end
    end

    player:update(dt)
    for idx,obj in ipairs(GAME_OBJECTS) do
        obj:update(dt)
        if obj.dead then
            table.remove(GAME_OBJECTS, idx)
        end
    end

    -- Passing resources arrays to the ghost element
    if player.appending then
        player.appending_element.nutriments =   current_level.nutriments


    end


    for idx,p in ipairs(PARTICLES) do
        p:update(dt)
    end

    --Updating GUI elements
    for idx,gui in ipairs(GUI) do
        gui:update(dt)
        if not gui:isAlive() then
            table.remove(GUI, idx)
        elseif gui:testPoint(m_x, m_y) then
            gui:onHover(m_x, m_y)
        end
    end

end

function growing:draw()
    love.graphics.setFont(standard_font)
    love.graphics.draw(sky, 0, 0)


    --Drawing _moving_ world
    seedcam:attach()
    for idx,obj in ipairs(ENV_OBJECTS) do
        obj:draw()
    end

    player:draw()
    for idx,obj in ipairs(GAME_OBJECTS) do
        obj:draw()
    end

    for idx,p in ipairs(PARTICLES) do
        love.graphics.draw(p, 0, 0)
    end

    seedcam:detach()
    for i, gui in pairs(GUI) do
        gui:draw()
    end

    watermark()
    if DEBUG then
        debug_draw_stats(seedcam)
    end
    --DRAWING STATIC HUD
    --nope
end

function growing:mousereleased(m_x, m_y, button)
    x,y =   seedcam:mousepos()

    -- If left button is clicked
    if button == "l" then
        click_handled   =   false
        --Check if a gui element is clicked
        for i,gui in ipairs(GUI) do
            if gui:onClick(m_x,m_y) then
                click_handled   =   true
            end
        end

        --If clicking on any player element
        if not click_handled then
            if player.appending then
                print("Appending something...")
                element.selected        =   false
                player.appending_element:confirm()
                player:connectNutriment(current_level.nutriments)
                player.appending_element=   nil
                player.appending        =   false

            else
                --~ if element then  end --passing old selected element to false
                if element then element.selected = false  end
                print("testing player...")
                element =   player:testPoint(x, y)
                if element then
                    radial  =   RadialMenu(element)
                    radial:setItems(element:getActions())
                    element.selected = true
                    radial:setPosition(m_x, m_y)
                    table.insert(GUI, radial)
                    radial  =   nil
                end
            end
        end
    end

    -- If mouse is right clicked
    if button == "r" then
        if element then element.selected = false  end
        if player.appending_element then
            print("Cancelling element")
            player.appending_element:cancel()
        end
    end
end

function growing:keypressed(key, unicode)
    if key == "escape" then
        love.event.quit()
    end

    if key == "t" then
        if player.transparent then
            player:setTransparent(false)
        else
            player:setTransparent(true)
        end
    end
end



